#ifdef __USAGE
%C - log process deaths (QNX)

%C	[-f file] [-r] [-a] &

Options:
 -f file               Append to this file when logging process deaths.
                       Default is write to stdout.
 -r                    Include register information in death log.
 -a                    All deaths, not just abnormal ones.
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <pwd.h>
#include <time.h>
#include <sys/kernel.h>
#include <sys/proc_msg.h>
#include <sys/psinfo.h>
#include <sys/seginfo.h>
#include <sys/debug.h>
#include <sys/dumper.h>
#include <sys/stat.h>

/*
 * Options.
 */
char *dumpfile;
long sigbits =
    (1L << SIGILL-1) |
    (1L << SIGTRAP-1)|
    (1L << SIGIOT-1) |
    (1L << SIGABRT-1)|
    (1L << SIGFPE-1) |
    (1L << SIGBUS-1) |
    (1L << SIGSEGV-1)|
    (1L << SIGEMT-1) ;


struct _dumper_hdr  hdr;
int dumpregs        = 0;
int dumpall         = 0;

struct {
    struct _sysmsg_hdr      hdr;
    struct _sysmsg_death    data;
} msg;


void shutdown() {
	pid_t pid;
	unsigned short msgdown = EOK;

	qnx_pflags(0, _PPF_INFORM, 0, 0);
	while((pid = Creceive(0, 0, 0)) != -1  &&  pid != 0)
		Reply(pid, &msgdown, sizeof(msgdown));

	exit(0);
	}


main(argc, argv)
int argc;
char *argv[];
    {
    int opt;
    pid_t pid;

    if(need_usage(argv))
        print_usage( argv );

    dumpfile = NULL;

    while ((opt = getopt(argc, argv, "f:ra")) != -1) {
        switch(opt) {
        case 'f':
            dumpfile = optarg;
            break;
		case 'r':
            dumpregs = 1;
            break;
		case 'a':
            dumpall = 1;
            break;
            }
        }

	for(opt = _SIGMIN ; opt < _SIGMAX ; ++opt)
		signal(opt, &shutdown);
	signal(SIGPWR, SIG_IGN);

	if (qnx_pflags(_PPF_INFORM, _PPF_INFORM, 0, 0) != 0) {
		perror("dumplog");
		exit(EXIT_FAILURE);
	}

	setpgid(0, 0); /* be pgrp leader */
    close(0);      /* close up stdin */
    if ( dumpfile != NULL ) close(1);
    close(2);      /* close up stderr */
    fcloseall();   /* to be safe ... close any open files */

    for(;;) {
        memset(&hdr, 0, sizeof(hdr));
        pid = Receive(0, &msg, sizeof(msg));

        if(msg.hdr.type == _SYSMSG
        && msg.hdr.subtype == _SYSMSG_SUBTYPE_DEATH
        && pidok(msg.data.signum)
        && qnx_osinfo((nid_t)0, &hdr.osdata) == 0
        && qnx_psinfo(PROC_PID, pid, &hdr.psdata, 0, 0) == pid
        &&(hdr.psdata.flags & (_PPF_VID|_PPF_MID)) == 0)
            dump_process(pid);

        Reply(pid, &msg, 0);
        }
    }


pidok(signum)
int signum;
	{
	if(dumpall || (signum  &&  (sigbits & (1L << signum-1))))
		return(1);
	return(0);
	}

dump_process(pid)
pid_t pid;
    {
    FILE    *fp = NULL;
    struct tm now;
    time_t ltime;
    char buf[26];

    if ( dumpfile == NULL ) 
       fp = stdout;
    else
       fp = fopen(dumpfile,"a");

    if ( !fp )
       return(-1);

    fprintf(fp,"%s ",hdr.psdata.un.proc.name);
    time ( &ltime );
    _localtime( &ltime, &now );
    fprintf(fp,":died on %24.24s :", _asctime( &now, buf ));
    if ( msg.data.signum ) {
       fprintf(fp,"Signal ");
       printsignal(fp, msg.data.signum);
       }
    else
       fprintf(fp,"Normal death");

    fprintf(fp,"\n");

    if ( dumpregs ) {
       if(__qnx_debug_xfer(PROC_PID, pid, _DEBUG_REG_RD, &hdr.reg,
                           sizeof(hdr.reg), 0, 0) != -1)
            fprintf(fp,"    Registers: cs:ip=%08lx:%08lx ds=%08lx\n",
                        hdr.reg.cs, hdr.reg.ip, hdr.reg.ds);
       }

    fflush(fp);
    if ( dumpfile != NULL )
       fclose(fp);
	return(0);
    }

struct system_defines { int num;
                        char *manifest;
                      };

struct system_defines signaldefines[] = {
   {    1,"SIGHUP"            }, 
   {    2,"SIGINT"            }, 
   {    3,"SIGQUIT"           }, 
   {    4,"SIGILL"            }, 
   {    5,"SIGTRAP"           }, 
   {    6,"SIGIOT"            }, 
   {    7,"SIGEMT"            }, 
   {    8,"SIGFPE"            }, 
   {    9,"SIGKILL"           }, 
   {   10,"SIGBUS"            }, 
   {   11,"SIGSEGV"           }, 
   {   12,"SIGSYS"            }, 
   {   13,"SIGPIPE"           }, 
   {   14,"SIGALRM"           }, 
   {   15,"SIGTERM"           }, 
   {   16,"SIGUSR1"           }, 
   {   17,"SIGUSR2"           }, 
   {   18,"SIGCHLD"           }, 
   {   19,"SIGPWR"            }, 
   {   20,"SIGWINCH"          }, 
   {   21,"SIGURG"            }, 
   {   22,"SIGPOLL/SIGIO"     }, 
   {   23,"SIGSTOP"           }, 
   {   24,"SIGTSTP"           }, 
   {   25,"SIGCONT"           },
   {   26,"SIGTTIN"           }, 
   {   27,"SIGTTOU"           }, 
   {   28,"SIGDEV"            }, 
   {    0,NULL                } 
   };

printsignal(FILE *fp, int signum)
{
int i = 0;
int keepgo = 1;

while( keepgo && signaldefines[i].manifest != NULL ) {
     if ( signaldefines[i].num == signum ) {
        fprintf(fp,"%s",signaldefines[i].manifest);
        keepgo = 0;
        }
     i++;
     }
}

