
/* an example using qnx_osstat()

   This program is provided as an example of how 'sac' gets its information
   on CPU usage.
   
   Note the following things:
   1. If you want to use cpu_load, you MUST send the special message to
      Proc described below. This 'seeds' Proc with an inertia value that it
      uses to calculate the averages.
      The 'sac' program (with -i option) sends this special message.
      You only have to set the inertia up once... anything non-zero will tell
      Proc to start calculating.
   2. If inertia is used, there is a slight bit of overhead on a timer tick
      when Proc does its calculations. It's extremely small, but, if you have
      a CPU contrained system you should be aware of this.
   3. If no inertia is set (it's zero), then cpu_load will return only zeros.
      The only array that will be valid at that time is cpu_ready[]. It will
      contain the number of ready processes (the depth of the ready queue) at
      each priority.

That's it!
Compile this program as:  cc osstat.c -o osstat

This program is really barebones. It just does a printf on the values returned.
You could take this output and timestamp it if you want, store it, keep
averages etc.
*/


#include <stdio.h>
#include <stdlib.h>
#include <sys/osstat.h>
#include <sys/kernel.h>
#include <sys/sys_msg.h>
#include <sys/trace.h>

#include <unistd.h>
#include <string.h>

main()
{
struct _osstat osdata;
unsigned short i, maxv;
union {
struct {
    struct _sysmsg_hdr hdr;
    struct _sysmsg_trace data;
} s;
struct {
    struct _sysmsg_hdr_reply hdr;
    struct _sysmsg_trace_reply data;
} r;
} msg;

pid_t vid = PROC_PID;
int inertia;			
long msec = 1000;			/* Pause between snaps */

    inertia = 3;    /* default */

    /* special message to Proc */
    if (inertia != -1) {
	if (inertia > 12)
	    inertia = 12;
	msg.s.hdr.type = _SYSMSG;
	msg.s.hdr.subtype = _SYSMSG_SUBTYPE_TRACE;
	msg.s.data.op = _TRACE_SET_POINT;
	msg.s.data.point = _TRACE_PROC_SAC;
	msg.s.data.value = inertia ? 1 << (inertia + 3) : 0;
	Send(vid, &msg.s, &msg.r, sizeof msg.s, sizeof msg.r);
    } else {
	msg.s.hdr.type = _SYSMSG;
	msg.s.hdr.subtype = _SYSMSG_SUBTYPE_TRACE;
	msg.s.data.op = _TRACE_GET_POINT;
	msg.s.data.point = _TRACE_PROC_SAC;
	Send(vid, &msg, &msg, sizeof msg.s, sizeof msg.r);
	inertia = 0;
	if (msg.r.data.value)
	    for (inertia = -3; (msg.r.data.value & 1) == 0; ++inertia)
		msg.r.data.value >>= 1;

	if (inertia == 0) {
	    printf("System activity disabled on this node.\n");
	}
    }

if ( inertia ) 
   printf("Inertia enabled\n");
else
   printf("Ready queue depth: no inertia\n");

    while(1) {
	    if (qnx_osstat(0, &osdata) == -1) {
           perror("qnx osstat");
           exit(EXIT_FAILURE);
           }

        for ( maxv=i=0; i < 32; ++i ) 
           maxv += osdata.cpu_load[i];

        if ( maxv == 0 )
           maxv = 1;

        if ( inertia ) {
           for ( i=0; i < 32; ++i ) {
               printf("%2u ", (osdata.cpu_load[i]*100)/maxv);
               }
           }
        else {
           /* the information returned is the number of processes ready
              at each priority (depth of the ready queue at each priority)
            */
           for ( i=0; i < 32; ++i ) {
               printf("%u ", osdata.cpu_ready[i]);
               }
           }
        printf("\n");
        fflush(stdout);
	    delay(msec);
        }
}

